#!/usr/bin/env python3
##########################################################
# hmmscan parser for dbCAN meta server
#
# Based off the hmmscan parser used in the dbCAN server,
# written by Dr. Yin
#
# Written by Tanner Yohe under the supervision
# of Dr. Yin in the YinLab at NIU.
#
# Updated by Le Huang from tips the contributor WATSON Mick <mick.watson@roslin.ed.ac.uk>,
# Thank you!
#
# INPUT
# python hmmscan-parser-dbCANmeta.py [inputFile] [eval] [coverage]
# eval and coverage are optional, inputFile is required
# -updating info:
# -adds pid for every subprocess to make codes robust.
# Last updated: 1/10/19
###########################################################


from subprocess import call
import sys
import os 


eval = 1e-15
coverage = 0.35

if len(sys.argv) > 1:
	inputFile = sys.argv[1]
else:
	print ("Please give a hmmscan output file as the first command")
	exit()

if len(sys.argv) > 3:
	eval = float(sys.argv[2])
	coverage = float(sys.argv[3])

tmpfile = "temp." + str(os.getpid())

call("cat "+inputFile+"  | grep -v '^#' | awk '{print $1,$3,$4,$6,$13,$16,$17,$18,$19}' | sed 's/ /\t/g' | sort -k 3,3 -k 8n -k 9n | perl -e 'while(<>){chomp;@a=split;next if $a[-1]==$a[-2];push(@{$b{$a[2]}},$_);}foreach(sort keys %b){@a=@{$b{$_}};for($i=0;$i<$#a;$i++){@b=split(/\t/,$a[$i]);@c=split(/\t/,$a[$i+1]);$len1=$b[-1]-$b[-2];$len2=$c[-1]-$c[-2];$len3=$b[-1]-$c[-2];if($len3>0 and ($len3/$len1>0.5 or $len3/$len2>0.5)){if($b[4]<$c[4]){splice(@a,$i+1,1);}else{splice(@a,$i,1);}$i=$i-1;}}foreach(@a){print $_.\"\n\";}}' > " + tmpfile, shell=True)


with open(tmpfile) as f:
	for line in f:
		row = line.rstrip().split('\t')
		row.append(float(int(row[6])-int(row[5]))/int(row[1]))
		if float(row[4]) <= eval and float(row[-1]) >= coverage:
			print('\t'.join([str(x) for x in row]))

call(['rm', tmpfile])



# resolve path to Hotpep data file,
# both when installed as a package, and in development

import os.path
import pkg_resources

def hotpep_data_path(*args):
    return pkg_resources.resource_filename('Hotpep', os.path.join(*args))



#!/usr/bin/env python3
#Runs parallel_group_many_proteins_many_patterns.rb for many organisms and types of proteins
#####################################
##Revised by Le Huang on 12/24/2018##
#####################################
from subprocess import call
import os
import os.path
import sys

from Hotpep.hotpep_data import hotpep_data_path

###
#train_many_organisms_many_families.py [inputFolder] [threads] [hits] [freq]
###

organism_array = ["Chaetomium_globosum_cbs_148_51"]
#Chaetomium_globosum_cbs_148_51 fungus_fungus

cazyme_array = ["CE", "GH", "AA", "PL", "GT", "CBM"]
threads = 1
peptide_length = 6 #length of conserved peptides
hit_cut_off = 3 #number of conserved peptides necessary to classify a protein
freq_cut_off = 1.0 #minimum sum of frequencies necessary to classify a protein
list_multidomain_enzymes = "yes"

if len(sys.argv) > 1:
	organism_array = [sys.argv[1]]
if len(sys.argv) > 2:
	threads = int(sys.argv[2])
if len(sys.argv) > 4:
	hit_cut_off = int(sys.argv[3])
	freq_cut_off = float(sys.argv[4])
#Start Add by Le Huang 12/24/2018
if os.path.exists(organism_array[0]+'/Results/output.txt'):
	call(['rm', organism_array[0]+'/Results/output.txt'])
#End Add by Le Huang 12/24/2018

##Start Delete by Le Huang
#try:
# 	call(['rm', organism_array[0]+'/Results/output.txt'])
# except:
# 	pass
## End Delete by Le Huang
for protein_dir_name in organism_array:
	print("Screening "+protein_dir_name+" for")
	for cazy_class in cazyme_array:
		print(cazy_class)
		peptide_dir_name = hotpep_data_path("CAZY_PPR_patterns", cazy_class)
		variables =  [threads, protein_dir_name, peptide_dir_name, peptide_length, hit_cut_off, freq_cut_off]
		call("parallel_group_many_proteins_many_patterns_noDNA.py "+" ".join(str(x) for x in variables), shell=True)
		#call(["add_functions_orf.py", protein_dir_name, peptide_dir_name])
		var1 = 1
		while var1 <= threads:
			try:
				os.remove(protein_dir_name+"/thread"+str(var1)+".txt")
			except:
				pass
			var1 += 1

	if list_multidomain_enzymes == "yes":
		call("list_multidomain_proteins.py "+protein_dir_name+" "+"_".join(cazyme_array), shell=True)
print("\nScreened\n"+"\n".join(organism_array))
print("for proteins of the types\n"+", ".join(cazyme_array))



#!/usr/bin/env python3
import sys
import os
import os.path

from Hotpep.hotpep_data import hotpep_data_path

protein_collection = hotpep_data_path("fungus_fungus/Chaetomium_thermophilum")
if len(sys.argv) > 1:
	protein_collection = sys.argv[1].replace("?", " ")
cazyme_array = ["AAexp"]
if len(sys.argv) > 2:
	cazyme_array = sys.argv[2].split("_")

assension_family_hash = {}

for cazy_class in cazyme_array:
	if os.path.exists(protein_collection+"/"+cazy_class):
		for filename in os.listdir(protein_collection+"/"+cazy_class):
			if "summary" not in filename and filename.endswith(".txt"):
				fam = filename[:-4]
				array = open(protein_collection+"/"+cazy_class+"/"+filename, 'r').readlines()
				array = array[1:]
				for hit in array:
					arr = hit.split('/t')
					if len(arr) >1:
						if arr[1] not in assension_family_hash:
							assension_family_hash[arr[1]] = []
						assension_family_hash[arr[1]].append(fam+"_"+arr[0])

for key in assension_family_hash:
	if len(assension_family_hash[key]) < 2:
		assension_family_hash.pop(key, 0)
array = assension_family_hash
if len(array) > 0:
	with open(protein_collection+"/summary_multidomain_enzymes.txt", "w") as out:
		out.write("Protein\tDomains\n")
		for arr in array:
			out.write(arr[0]+'\t'+'\t'.join(arr[1])+'\n')
			



#!/usr/bin/env python3
import sys
import os
import os.path
from subprocess import call
import natsort

from Hotpep.hotpep_data import hotpep_data_path

protein_dir_name = hotpep_data_path("fungus_fungus")
if len(sys.argv) > 1:
	protein_dir_name = sys.argv[1].replace("?", " ")
function_significance_limit = 2 #Normally 2 for new patterns (no proteins with this function in group) and 10 for old patterns (sum freq conserved peptides for function in group ) 
score_file = "_group_ec"

peptide_dir_name = hotpep_data_path("CAZY_PPR_patterns/GH")
if len(sys.argv) > 2:
	peptide_dir_name = sys.argv[2].replace("?", " ")
	

fam_group_score_hash = {}
with open(peptide_dir_name+"/fam_list.txt", 'r') as f: ####
	for line in f:
		fam = line.rstrip()
		try:
			with open(peptide_dir_name+"/"+fam+"/"+fam+score_file+".txt", 'r') as f2:
				group_func_hash = {}
				for row in f2:
					row = row.rstrip()
					arr = row.split('\t')
					if len(arr) > 1:
						group_func_hash[arr[0]] = arr[1].replace("*", "x")
			fam_group_score_hash[fam] = group_func_hash
		except:
			pass
			
output_dir_name = protein_dir_name+"/"+peptide_dir_name.split("/")[-1]+"/functions"
if not os.path.exists(output_dir_name):
	call(["mkdir", output_dir_name])

class Protein():
	def __init__(self, sequ):
		self.seq = sequ.upper()
		self.dna = None
		self.name = None
		self.peptides = None
		self.hits = None
		self.freq = None
		self.group = None
		self.fam = None
		self.accession = None
		self.neighbor_seqs = None
		self.orf_hits = None
		self.orf_freq = None
		self.function = None
		self.functions_array = None
	def __repr__(self):
		return '{}: {} {} {}'.format(self.name, self.freq, self.hits, self.function)
		
function_summary_hash = {}
protein_array = []
with open(protein_dir_name+"/"+peptide_dir_name.split("/")[-1]+"/summary.txt", 'r') as f:
	next(f)
	for line in f:
		line = line.rstrip()
		if int(line.split("\t")[1]) > 0:
			with open(protein_dir_name+"/"+peptide_dir_name.split("/")[-1]+"/"+line.split("\t")[0]+".txt", 'r') as f2:
				next(f2)
				p_old = Protein("load")
				p_old.neighbor_seqs = []
				for row in f2:
					row = row.rstrip()
					arr = row.split("\t")
					p = Protein(arr[4])
					p.fam = line.split("\t")[0]
					p.group = arr[0]
					p.name = arr[1]
					p.freq = float(arr[2].replace(",", "."))
					p.hits = int(arr[3])
					p.peptides = arr[6]
					p.neighbor_seqs = []
					p.functions_array = fam_group_score_hash[p.fam][p.group].split(", ")
					function_score = int(p.functions_array[0].split(":")[1].replace(",",""))
					if function_score >= function_significance_limit:
						p.function = p.functions_array[0].split(":")[0]
						if "*" in p.name:
							p_old.neighbor_seqs.append(p)
						else:
							if p_old.seq != "LOAD":
								protein_array.append(p_old)
							p.neighbor_seqs = []
							p_old = p
		if p_old.seq != "LOAD":
			protein_array.append(p_old)

for protein in protein_array:
	if protein.function == None:
		protein_array.delete(protein)
def sort_order(protein):
	func = int(protein.function.replace(".", "").replace("x", "0"))
	return (func, -protein.freq, -protein.hits)
	

if len(protein_array) > 0:
	protein_array.sort(key=sort_order)
	function = protein_array[0].function
	out = open(protein_dir_name+"/"+peptide_dir_name.split("/")[-1]+"/functions/"+function.replace(".","_")+".txt", 'w')
	out.write("fam\tgroup\tFunctions\tseq_no\tORF freq\tORF hits\tsequence\tlength\tpeptides\tFrequency\thits\tDNA\n")
	for p in protein_array:
		if p.function != function:
			function = p.function
			out.close()
			out = open(protein_dir_name+"/"+peptide_dir_name.split("/")[-1]+"/functions/"+function.replace(".","_")+".txt", 'w')
			out.write("fam\tgroup\tFunctions\tseq_no\tORF freq\tORF hits\tsequence\tlength\tpeptides\tFrequency\thits\tDNA\n")
		out.write(p.fam+'\t'+str(p.group)+'\t'+",".join(p.functions_array)+'\t'+p.name.split("|")[0]+'\t'+str(p.orf_freq).replace(".",",")+'\t'+str(p.orf_hits)+'\t'+p.seq+'\t'+str(len(p.seq))+'\t'+p.peptides+'\t'+str(p.freq).replace(".",",")+'\t'+str(p.hits)+'\t'+str(p.dna)+'\n')
		if len(p.neighbor_seqs) > 0:
			for n in neighbor_seqs:
				out.write(n.fam+'\t'+n.group+'\t'+",".join(n.functions_array)+'\t'+n.name.split("|")[0]+'\t'+n.orf_feq.replace(".",",")+'\t'+n.orf_hits+'\t'+n.seq+'\t'+len(n.seq)+'\t'+n.peptides+'\t'+n.freq.replace(".",",")+'\t'+n.hits+'\t'+n.dna+'\n')
	out.close()
	
	function_hash = {}
	function = ""
	for p in protein_array:
		if p.function != function:
			function = p.function
			array = [p.fam]
			function_hash[function] = array
		else:
			function_hash[function].append(p.fam)
	out = open(protein_dir_name+"/"+peptide_dir_name.split("/")[-1]+"/functions/summary.txt", 'w')
	out.write("EC number\thits\tfamily distribution")
	#array = sorted(function_hash, key=lambda x: (x[0].replace(".","").replace("x","0")))
	array = function_hash
	for key in array: 
		out.write(key+'\t'+str(len(array[key])))
		fam_count_hash={}
		for x in array[key]:
			if x not in fam_count_hash:
				fam_count_hash[x] = 0
			fam_count_hash[x] += 1
		#out_arr = natsort.natsorted(fam_count_hash)
		out_arr = fam_count_hash
		for arr in out_arr:
			out.write("\t"+str(out_arr[arr])+" "+arr)
		out.write("\n")	
	out.close()
				



#!/usr/bin/env python3
#########################################################
# dbCAN2 Driver Script (Stand Alone Version)
#
# Written by Tanner Yohe in the Yin Lab at NIU
# Revised by Le Huang in the Zhang Lab at NKU
# Updated by Mohamad Majd Raslan in the Yin Lab at NIU
# Updated by Wei Li created table
# Updated by Le Huang at NKU
# Last updated 09/08/19
# updated information[Le Huang]: 1. suitable for python3 users 2. fixed the bugs when the count of sequences is small.
# Accepts user input
# Predicts genes if needed
# Runs input against HMMER, DIAMOND, and Hotpep
# Optionally predicts CGCs with CGCFinder
# Creats an overview table using output files from core
# tools from Hotpep.out,hmmer.out and diamond.out
##########################################################
from subprocess import Popen, call, check_output
import os
import argparse
import sys

'''
def some functions
'''
def runHmmScan(outPath, hmm_cpu, dbDir, hmm_eval, hmm_cov, db_name):
    hmmer = Popen(['hmmscan', '--domtblout', '%sh%s.out' % (outPath, db_name), '--cpu', hmm_cpu, '-o', '/dev/null', '%s%s.hmm' % (dbDir,db_name), '%suniInput' % outPath])
    hmmer.wait()
    call('hmmscan-parser.py %sh%s.out %s %s > %s%s.out'%(outPath, db_name, hmm_eval, hmm_cov, outPath, db_name), shell=True)
    if os.path.exists('%sh%s.out' % (outPath, db_name)):
        call(['rm', '%sh%s.out' % (outPath, db_name)])

parser = argparse.ArgumentParser(description='dbCAN2 Driver Script')


parser.add_argument('inputFile', help='User input file. Must be in FASTA format.')
parser.add_argument('inputType', choices=['protein', 'prok', 'meta'], #protein=proteome, prok=prokaryote nucleotide, meta=metagenome nucleotide
                    help='Type of sequence input. protein=proteome; prok=prokaryote; meta=metagenome') 
parser.add_argument('--cluster', '-c', help='Predict CGCs via CGCFinder. This argument requires an auxillary locations file if a protein input is being used')
parser.add_argument('--dbCANFile',default="dbCAN.txt", help='Indicate the file name of HMM database such as dbCAN.txt, please use the newest one from dbCAN2 website.')
parser.add_argument('--dia_eval', default=1e-102,type=float, help='DIAMOND E Value')
parser.add_argument('--dia_cpu', default=4, type=int, help='Number of CPU cores that DIAMOND is allowed to use')
parser.add_argument('--hmm_eval', default=1e-15, type=float, help='HMMER E Value')
parser.add_argument('--hmm_cov', default=0.35, type=float, help='HMMER Coverage val')
parser.add_argument('--hmm_cpu', default=4, type=int, help='Number of CPU cores that HMMER is allowed to use')
parser.add_argument('--hotpep_hits', default=6, type=int, help='Hotpep Hit value')
parser.add_argument('--hotpep_freq', default=2.6, type=float, help='Hotpep Frequency value')
parser.add_argument('--hotpep_cpu', default=3, type=int, help='Number of CPU cores that Hotpep is allowed to use')
parser.add_argument('--tf_eval', default=1e-4, type=float, help='tf.hmm HMMER E Value')
parser.add_argument('--tf_cov', default=0.35, type=float, help='tf.hmm HMMER Coverage val')
parser.add_argument('--tf_cpu', default=1, type=int, help='tf.hmm Number of CPU cores that HMMER is allowed to use')
parser.add_argument('--stp_eval', default=1e-4, type=float, help='stp.hmm HMMER E Value')
parser.add_argument('--stp_cov', default=0.3, type=float, help='stp.hmm HMMER Coverage val')
parser.add_argument('--stp_cpu', default=1, type=int, help='stp.hmm Number of CPU cores that HMMER is allowed to use')
parser.add_argument('--out_pre', default="", help='Output files prefix')
parser.add_argument('--out_dir', default="output", help='Output directory')
parser.add_argument('--db_dir', default="db/", help='Database directory')
parser.add_argument('--cgc_dis', default=2, help='CGCFinder Distance value')
parser.add_argument('--cgc_sig_genes', default='tp', choices=['tp', 'tf','all'], help='CGCFinder Signature Genes value')
parser.add_argument('--tools', '-t', nargs='+', choices=['hmmer', 'diamond', 'hotpep', 'all'], default='all', help='Choose a combination of tools to run')
parser.add_argument('--use_signalP', default=False, type=bool, help='Use signalP or not, remember, you need to setup signalP tool first. Because of signalP license, Docker version does not have signalP.')
parser.add_argument('--gram', '-g', choices=["p","n","all"], default="all", help="Choose gram+(p) or gram-(n) for proteome/prokaryote nucleotide, which are params of SingalP, only if user use singalP")
args = parser.parse_args()



####
#run_dbcan.py [inputFile] [inputType]
####

##########################
# Begin Setup and Input Checks

dbDir = args.db_dir
prefix = args.out_pre
outDir = args.out_dir

if not dbDir.endswith("/") and len(dbDir) > 0:
    dbDir += "/"

if not outDir.endswith("/") and len(outDir) > 0:
    outDir += "/"

outPath = outDir + prefix
auxFile = ""
inputFile = args.inputFile
inputType = args.inputType
find_clusters = False
if args.cluster != None:
    find_clusters = True
    if inputType == "protein":
        auxFile = args.cluster
    else:
        auxFile = '%sprodigal.gff'%outPath

if not os.path.isdir(dbDir):
    print(dbDir , "ERROR: The database directory does not exist")
    exit()

if not os.path.isfile(dbDir+'CAZy.dmnd'):
    print("ERROR: No CAZy DIAMOND database found. \
    Please make sure that your CAZy DIAMOND databased is named 'CAZy.dmnd' and is located in your database directory")
    exit()

if not os.path.isfile(dbDir + args.dbCANFile):
    print("ERROR: No dbCAN HMM database found. \
    Please make sure that your dbCAN HMM database is named 'dbCAN-HMMdb-V8.txt' or the newest one, has been through hmmpress, and is located in your database directory")
    exit()

if not os.path.isdir(outDir):
    call(['mkdir', outDir])

if find_clusters and inputType == "protein":
    if len(auxFile) > 0:
        print(auxFile)
        if not os.path.isfile(auxFile):
                print("ERROR: It seems that the auxillary filename that you provided does not exist, or is not a file")
                exit()
    else:
        print("ERROR: Please provide an auxillary input file with the position of each gene. This file can either be in BED or GFF format")
        exit()
tools = [True, True, True] #DIAMOND, HMMER, Hotpep
if args.tools != 'all':
    if 'diamond' not in args.tools:
        tools[0] = False
    if 'hmmer' not in args.tools:
        tools[1] = False
    if 'hotpep' not in args.tools:
        tools[2] = False


# End Setup and Input Checks
#########################
#########################
# Begin Gene Prediction Tools
if inputType == 'prok':
    call(['prodigal', '-i', inputFile, '-a', '%suniInput'%outPath, '-o', '%sprodigal.gff'%outPath, '-f', 'gff', '-q'])
if inputType == 'meta':
    call(['prodigal', '-i', inputFile, '-a', '%suniInput'%outPath, '-o', '%sprodigal.gff'%outPath, '-f', 'gff', '-p', 'meta','-q'])
#Proteome
if inputType == 'protein':
    call(['cp', inputFile, '%suniInput'%outPath])

# End Gene Prediction Tools
#######################
# Begin SignalP
if args.use_signalP:
    print("***************************0. SIGNALP start*************************************************\n\n")
    if args.gram == "p" or args.gram=="all":
        signalpos = Popen('signalp -t gram+ %suniInput > %ssignalp.pos' % (outPath, outPath), shell=True)
    if args.gram == "n" or args.gram == "all":
        signalpneg = Popen('signalp -t gram- %suniInput > %ssignalp.neg' % (outPath, outPath), shell=True)

# End SignalP
#######################
# Begin Core Tools

if tools[0]:
    # diamond blastp -d db/CAZy -e 1e-102 -q output_EscheriaColiK12MG1655/uniInput -k 1 -p 2 -o output_EscheriaColiK12MG1655/diamond1.out -f 6
    print("\n\n***************************1. DIAMOND start*************************************************\n\n")
    os.system('diamond blastp -d %sCAZy -e %s -q %suniInput -k 1 -p %d -o %sdiamond.out -f 6'%(dbDir, str(args.dia_eval), outPath, args.dia_cpu, outPath))
    # diamond = Popen(['diamond', 'blastp', '-d', '%sCAZy.dmnd' % dbDir, '-e', str(args.dia_eval), '-q', '%suniInput' % outPath, '-k', '1', '-p', str(args.dia_cpu), '-o', '%sdiamond.out'%outPath, '-f', '6'])
    print("\n\n***************************1. DIAMOND end***************************************************\n\n")

if tools[1]:
    print("\n\n***************************2. HMMER start*************************************************\n\n")
    os.system(f"hmmscan --domtblout {outPath}h.out --cpu {args.hmm_cpu} -o /dev/null {dbDir}{args.dbCANFile} {outPath}uniInput ")
    print("\n\n***************************2. HMMER end***************************************************\n\n")
    call(f"hmmscan-parser.py {outPath}h.out {str(args.hmm_eval)} {str(args.hmm_cov)} > {outPath}hmmer.out", shell=True)
    if os.path.exists(f"{outPath}h.out"):
        call(['rm', f"{outPath}h.out"])

if tools[2]:
    count = int(check_output(f"tr -cd '>' < {outPath}uniInput | wc -c" , shell=True))    #number of genes in input file
    numThreads = args.hotpep_cpu if count >= args.hotpep_cpu else count   #number of cores for Hotpep to use, revised by Le Huang 12/17/2018
    count_per_file = count / numThreads      #number of genes per core
    directory = "input_" + str(os.getpid())
    if not os.path.exists(f"Hotpep/{directory}"):
        os.makedirs(f"Hotpep/{directory}")
    num_files = 1
    num_genes = 0
    out = open(f"Hotpep/{directory}/orfs{str(num_files)}.txt", "w")
    with open(f"{outPath}uniInput", "r") as f:
        for line in f:
            if line.startswith(">"):
                num_genes += 1
                if num_genes > count_per_file and num_files != numThreads:
                    out.close()
                    num_files += 1
                    num_genes = 0
                    out = open(f"Hotpep/{directory}/orfs{str(num_files)}.txt", "w")
            out.write(line)
    out.close()
    os.chdir("Hotpep/")
    print("\n\n***************************3. HotPep start***************************************************\n\n")
    os.system(f"train_many_organisms_many_families.py {directory} {numThreads} {args.hotpep_hits} {args.hotpep_freq}")
    os.chdir('../')
    print("\n\n***************************3. hotPep end***************************************************\n\n")
    os.system(f"cp Hotpep/{directory}/Results/output.txt {outPath}Hotpep.out")
    os.system(f"rm -r Hotpep/{directory}")

# End Core Tools
########################
# Begin Adding Column Headers

if tools[2]:
    with open(outPath+'Hotpep.out') as f:
        with open(outPath+'temp', 'w') as out:
            out.write('CAZy Family\tPPR Subfamily\tGene ID\tFrequency\tHits\tSignature Peptides\tEC number\n')
            for line in f:
                more_information = line.split("\t")
                out.write(line)
    call(['mv', outPath+'temp', outPath+'Hotpep.out'])
if tools[1]:
    with open(outDir+prefix+'hmmer.out') as f:
        with open(outDir+prefix+'temp', 'w') as out:
            out.write('HMM Profile\tProfile Length\tGene ID\tGene Length\tE Value\tProfile Start\tProfile End\tGene Start\tGene End\tCoverage\n')
            for line in f:
                out.write(line)
    call(['mv', outDir+prefix+'temp', outDir+prefix+'hmmer.out'])
if tools[0]:
    with open(outDir+prefix+'diamond.out') as f:
        with open(outDir+prefix+'temp', 'w') as out:
            out.write('Gene ID\tCAZy ID\t% Identical\tLength\tMismatches\tGap Open\tGene Start\tGene End\tCAZy Start\tCAZy End\tE Value\tBit Score\n')
            for line in f:
                out.write(line)
    call(['mv', outDir+prefix+'temp', outDir+prefix+'diamond.out'])

# End Adding Column Headers
########################
# Begin CGCFinder

if find_clusters:
    print("*****************************CGC-Finder start************************************")

########################
# Begin TF,TP, STP prediction
    '''
    previous tf uses diamond, Now tf-1 and tf-2 uses hmmer
    tf hmmer
    '''
    #call(['diamond', 'blastp', '-d', dbDir+'tf_v1/tf.dmnd', '-e', '1e-10', '-q', '%suniInput' % outPath, '-k', '1', '-p', '1', '-o', outDir+prefix+'tf.out', '-f', '6'])
    runHmmScan(outPath, str(args.tf_cpu), dbDir, str(args.tf_eval), str(args.tf_cov), "tf-1")
    runHmmScan(outPath, str(args.tf_cpu), dbDir, str(args.tf_eval), str(args.tf_cov), "tf-2")
    '''
    stp hmmer
    '''
    runHmmScan(outPath, str(args.stp_cpu), dbDir, str(args.stp_eval), str(args.stp_cov), "stp")

    '''
    tp diamond
    '''
    call(['diamond', 'blastp', '-d', dbDir+'tcdb.dmnd', '-e', '1e-10', '-q', '%suniInput' % outPath, '-k', '1', '-p', '1', '-o', outDir+prefix+'tp.out', '-f', '6'])


    tp = set()
    tf = set()
    stp = set()

    tp_genes = {}
    tf_genes = {}
    stp_genes = {}

    with open("%stf-1.out" % outPath) as f:
        for line in f:
            row = line.rstrip().split('\t')
            tf.add(row[2])
            row[0] = "DBD-Pfam|" + row[0]
            if not row[2] in tf_genes:
                tf_genes[row[2]] = row[0]
            else:
                tf_genes[row[2]] += ',' + row[0]

    with open("%stf-2.out" % outPath) as f:
        for line in f:
            row = line.rstrip().split('\t')
            tf.add(row[2])
            row[0] = "DBD-SUPERFAMILY|" + row[0]
            if not row[2] in tf_genes:
                tf_genes[row[2]] = row[0]
            else:
                tf_genes[row[2]] += ',' + row[0]

    with open(outDir+prefix+'tp.out') as f:
        for line in f:
            row = line.rstrip().split('\t')
            tp.add(row[0])
            if not row[0] in tp_genes:
                tp_genes[row[0]] = row[1]
            else:
                tp_genes[row[0]] += ','+row[1]

    with open("%sstp.out" % outPath) as f:
        for line in f:
            row = line.rstrip().split('\t')
            stp.add(row[2])
            row[0] = "STP|" + row[0]
            if not row[2] in stp_genes:
                stp_genes[row[2]] = row[0]
            else:
                stp_genes[row[2]] += ',' + row[0]
# End TF and TP prediction
##########################
# Begine CAZyme Extraction
    cazyme_genes = {}
    dia = set()
    hot = set()
    hmm = set()
    if tools[0]:
        with open(outDir+prefix+'diamond.out') as f:
            next(f)
            for line in f:
                row = line.rstrip().split('\t')
                dia.add(row[0])
                if row[0] not in cazyme_genes:
                    cazyme_genes[row[0]] = set()
                cazyme_genes[row[0]].update(set(row[1].strip("|").split('|')[1:]))
    if tools[1]:
        with open(outDir+prefix+'hmmer.out') as f:
            next(f)
            for line in f:
                row = line.rstrip().split('\t')
                hmm.add(row[2])
                if row[2] not in cazyme_genes:
                    cazyme_genes[row[2]] = set()
                cazyme_genes[row[2]].add(row[0].split('.hmm')[0])
    if tools[2]:
        with open(outDir+prefix+'Hotpep.out') as f:
            next(f)
            for line in f:
                row = line.rstrip().split('\t')
                hot.add(row[2])
                if row[2] not in cazyme_genes:
                    cazyme_genes[row[2]] = set()
                cazyme_genes[row[2]].add(row[0])
    if tools.count(True) > 1:
        temp1 = hmm.intersection(hot)
        temp2 = hmm.intersection(dia)
        temp3 = dia.intersection(hot)
        cazyme = temp1.union(temp2, temp3)
    else:
        cazyme = hmm.union(dia, hot)
# End CAZyme Extraction
######################
# Begin GFF preperation

    if inputType == "prok" or inputType == "meta":   #use Prodigal GFF output
        with open(outDir+prefix+'prodigal.gff') as f:
            with open(outDir+prefix+'cgc.gff', 'w') as out:
                for line in f:
                    if not line.startswith("#"):
                        row = line.rstrip().split('\t')
                        num = row[-1].split(";")[0].split('_')[-1]
                        gene = row[0] + '_' + num
                        row[8] = ""
                        if gene in cazyme:
                            row[2] = "CAZyme"
                            row[8] = "DB="+'|'.join(cazyme_genes[gene])
                        elif gene in tf:
                            row[2] = "TF"
                            row[8] = "DB="+tf_genes[gene]
                        elif gene in tp:
                            row[2] = "TC"
                            row[8] = "DB="+tp_genes[gene]
                        elif gene in stp:
                            row[2] = "STP"
                            row[8] = "DB="+stp_genes[gene]
                        row[8] += ";ID="+gene
                        out.write('\t'.join(row)+'\n')
    else:  #user provided GFF/BED file
        gff = False
        with open(auxFile) as f:
            for line in f:
                if not line.startswith('#'):
                    if len(line.split('\t')) == 9:
                        gff = True
                        break
        if gff:  #user file was in GFF format
            with open(auxFile) as f:
                with open(outDir+prefix+'cgc.gff', 'w') as out:
                    for line in f:
                        if not line.startswith("#"):
                            row = line.rstrip().split('\t')
                            if row[2] == "CDS":
                                note = row[8].strip().split(";")
                                gene = ""
                                notes = {}
                                for x in note:
                                    temp = x.split('=')
                                    notes[temp[0]] = temp[1]
                                if "Name" in notes:
                                    gene = notes["Name"]
                                elif "ID" in notes:
                                    gene = notes["ID"]
                                else:
                                    continue
                                
                                if gene in cazyme:
                                    row[2] = "CAZyme"
                                    row[8] = "DB="+'|'.join(cazyme_genes[gene])
                                elif gene in tf:
                                    row[2] = "TF"
                                    row[8] = "DB="+tf_genes[gene]
                                elif gene in tp:
                                    row[2] = "TC"
                                    row[8] = "DB="+tp_genes[gene]
                                elif gene in stp:
                                    row[2] = "STP"
                                    row[8] = "DB=" + stp_genes[gene]
                                else:
                                    row[8] = ""
                                row[8] += ";ID="+gene
                                out.write('\t'.join(row)+'\n')
        else:  #user file was in BED format
            with open(auxFile) as f:
                with open(outDir+prefix+'cgc.gff', 'w') as out:
                    for line in f:
                        if line.startswith("track"):
                            continue
                        row = line.rstrip().split('\t')
                        outrow = ['.','.','.','.','.','.','.','.','']
                        gene = row[1]
                        if gene in cazyme:
                            outrow[2] = 'CAZyme'
                            outrow[8] = "DB="+'|'.join(cazyme_genes[gene])
                        elif gene in tf:
                            outrow[2] = 'TF'
                            outrow[8] =  "DB="+tf_genes[gene]
                        elif gene in tp:
                            outrow[2] = 'TC'
                            outrow[8] = "DB="+tp_genes[gene]
                        elif gene in stp:
                            outrow[2] = 'STP'
                            outrow[8] = "DB=" + stp_genes[gene]
                        else:
                            outrow[2] = 'CDS'
                        outrow[0] = row[0]
                        outrow[3] = row[2]
                        outrow[4] = row[3]
                        outrow[6] = row[4]
                        outrow[8] += ";ID="+gene
                        out.write('\t'.join(outrow)+'\n')
    # End GFF preperation
    ####################
    # Begin CGCFinder call

    call(['CGCFinder.py', outDir+prefix+'cgc.gff', '-o', outDir+prefix+'cgc.out', '-s', args.cgc_sig_genes, '-d', str(args.cgc_dis)])
    print("**************************************CGC-Finder end***********************************************")

    # End CGCFinder call
    # End CGCFinder
    ####################
# Begin SignalP combination
if args.use_signalP:
    print("Waiting on signalP")
    with open(outDir+prefix+'temp', 'w') as out:
        if args.gram == "all" or args.gram =="p":
            signalpos.wait()
            print("SignalP pos complete")

            with open(outDir+prefix+'signalp.pos') as f:
                for line in f:
                    if not line.startswith('#'):
                        row = line.split(' ')
                        row = [x for x in row if x != '']
                        if row[9] == 'Y':
                            out.write(line)
            call(['rm', outDir+prefix+'signalp.pos'])
        if args.gram == "all" or args.gram == "n":
            signalpneg.wait()
            print("SignalP neg complete")
            with open(outDir+prefix+'signalp.neg') as f:
                for line in f:
                    if not line.startswith('#'):
                        row = line.split(' ')
                        row = [x for x in row if x != '']
                        if row[9] == 'Y':
                            out.write(line)
            call(['rm', outDir+prefix+'signalp.neg'])
    call('sort -u '+outDir+prefix+'temp > '+outDir+prefix+'signalp.out', shell=True)
    call(['rm', outDir+prefix+'temp'])

# End SignalP combination
#######################
#######################
# start Overview
print ("Preparing overview table from hmmer, hotpep and diamond output...")
workdir= outDir+prefix
# a function to remove duplicates from lists while keeping original order
def unique(seq):
    exists = set()
    return [x for x in seq if not (x in exists or exists.add(x))]

# check if files exist. if so, read files and get the gene numbers
if tools[0]:
    arr_diamond = open(workdir+"diamond.out").readlines()
    diamond_genes = [arr_diamond[i].split()[0] for i in range(1, len(arr_diamond))] # or diamond_genes = []

if tools[1]:
    arr_hmmer = open(workdir+"hmmer.out").readlines()
    hmmer_genes = [arr_hmmer[i].split()[2] for i in range(1, len(arr_hmmer))] # or hmmer_genes = []

if tools[2]:
    arr_hotpep = open(workdir+"Hotpep.out").readlines()
    hotpep_genes = [arr_hotpep[i].split()[2] for i in range(1, len(arr_hotpep))]# or hotpep_genes = []

if args.use_signalP and (os.path.exists(workdir + "signalp.out")):
    arr_sigp = open(workdir+"signalp.out").readlines()
    sigp_genes = {}
    for i in range (0,len(arr_sigp)):
        row = arr_sigp[i].split()
        sigp_genes[row[0]] = row[4] #previous one is row[2], use Y-score instead from suggestion of Dongyao Li

##Catie Ausland edits BEGIN, Le add variable exists or not, remove duplicates from input lists
if not tools[0]:
    diamond_genes =[]
if not tools[1]:
    hmmer_genes   = []
if not tools[2]:
    hotpep_genes  =[]
if len(hotpep_genes) > 0:
    if (hotpep_genes[-1] == None):
        hotpep_genes.pop()
        hotpep_genes = unique(hotpep_genes)
        if 'hmmer_genes' in locals():
            hmmer_genes.pop()
            hmmer_genes = unique(hmmer_genes)
        if 'diamond_genes' in locals():
            diamond_genes.pop()
            diamond_genes = unique(diamond_genes)
## Catie edits END, Le add variable exists or not, remove duplicates from input lists

# parse input, stroe needed variables
if tools[0] and (len(arr_diamond) > 1):
    diamond_fams = {}
    for i in range (1,len(arr_diamond)):
        row = arr_diamond[i].split("\t")
        fam = row[1].strip("|").split("|")
        diamond_fams[row[0]] = fam[1:]


if tools[1] and (len(arr_hmmer) > 1):
    hmmer_fams = {}
    for i in range (1, len(arr_hmmer)):
        row = arr_hmmer[i].split("\t")
        fam = row[0].split(".")
        fam = fam[0]+"("+row[7]+"-"+row[8]+")"
        if(row[2] not in hmmer_fams):
            hmmer_fams[row[2]] = []
        hmmer_fams[row[2]].append(fam)

if tools[2] and (len(arr_hotpep) > 1) :
    hotpep_fams = {}
    for i in range (1,len(arr_hotpep)):
        row = arr_hotpep[i].split("\t")
        if(row[2] not in hotpep_fams):
            hotpep_fams[row[2]] = []
        hotpep_fams[row[2]].append(row[0]+"("+row[1]+")")

#overall table

all_genes = unique(hmmer_genes+hotpep_genes+diamond_genes)
with open(workdir+"overview.txt", 'w+') as fp:
    if args.use_signalP:
        fp.write("Gene ID\tHMMER\tHotpep\tDIAMOND\tSignalp\t#ofTools\n")
    else:
        fp.write("Gene ID\tHMMER\tHotpep\tDIAMOND\t#ofTools\n")
    for gene in all_genes:
        csv=[gene]
        num_tools = 0
        if tools[1] and arr_hmmer != None and (gene in hmmer_genes):
            num_tools += 1
            csv.append("+".join(hmmer_fams[gene]))
        else:
            csv.append("-")
        if tools[2] and arr_hotpep!= None and (gene in hotpep_genes):
            num_tools += 1
            csv.append("+".join(hotpep_fams[gene]))
        else:
            csv.append("-")
        if tools[0] and arr_diamond != None and (gene in diamond_genes):
            num_tools += 1
            csv.append("+".join(diamond_fams[gene]))
        else:
            csv.append("-")
        if args.use_signalP:
            if (gene in sigp_genes):
                csv.append("Y(1-"+sigp_genes[gene]+")")
            else:
                csv.append("N")
        csv.append(str(num_tools))
        temp = "\t".join(csv) + "\n"
        fp.write(temp)
print ("overview table complete. Saved as "+workdir+"overview.txt")
# End overview



#!/usr/bin/env python3
import sys
import re
import itertools
import os
from subprocess import call

from Hotpep.hotpep_data import hotpep_data_path

protein_dir_name = hotpep_data_path("fungus_fungus")
peptide_dir_name = hotpep_data_path("CAZY_PPR_patterns/GH")
thread_no = 1
peptide_length = 6 #length of conserved peptides
hit_cut_off = 3 #number of conserved peptides necessary to classify a protein
freq_cut_off = 1.0 #minimum sum of frequencies necessary to classify a protein


argc = len(sys.argv)
if argc > 1:
	thread_no = int(sys.argv[1])
if argc > 2:
	protein_dir_name = sys.argv[2].replace("?", " ")
if argc > 3:
	peptide_dir_name = sys.argv[3].replace("?", " ")
if argc > 4:
	peptide_length = int(sys.argv[4])
if argc > 5:
	hit_cut_off = int(sys.argv[5])
if argc > 6:
	freq_cut_off = float(sys.argv[6])
outf_name = protein_dir_name+"/thread"+str(thread_no)+".txt"

class Protein:
	def __init__(self, sequ):
		self.seq = sequ.upper()
		self.name = None
		self.peptides = None
		self.hits = None
		self.freq = None
		self.group = None
		self.classified = None
	
	def make_nmer_peptides(self, n):
		self.peptides = []
		seq = self.seq
		for x in range(n):
			for y in range(x, len(seq), n):
				self.peptides.append(seq[y:y+n])
		set(self.peptides)
		
class Group:
	def __init__(self, fami, numb):
		self.fam = fami
		self.number = numb
		self.pep_hash = {}
		self.hits = None
		self.freq = None



prot_array = []
#####################
seq = ""
name = ""
p = ""
first = True
with open(protein_dir_name+"/orfs"+str(thread_no)+".txt") as f:
        
	for line in f:
		if line.startswith(">"):
			if not first:
				p = Protein(seq)
				p.name = name
				p.make_nmer_peptides(peptide_length)
				prot_array.append(p)
				seq = ""
			name = line.rstrip()
			first = False
		else:
			seq += line.rstrip()
p = Protein(seq)
p.name = name
p.make_nmer_peptides(peptide_length)
prot_array.append(p)
##########
#f = open(protein_dir_name+"/orfs"+str(thread_no)+".txt", 'r').readlines()
#for x in range(len(f)):
	#if f[x].startswith(">"):
		#p = Protein(f[x+1].rstrip())
		#p.name = f[x].rstrip()
		#p.make_nmer_peptides(peptide_length)
		#prot_array.append(p)
###################
pep_list_array = []
try:
	pep_list_array = open(peptide_dir_name+"/fam_list.txt", 'r').readlines()
except:
	pep_list_array = open(peptide_dir_name+"/large_fams.txt", 'r').readlines()
with open(outf_name, 'w') as out:
	for fam in pep_list_array:
		protein_hits = 0
		pep_group_array = []
		fam_all_peps_array = []
		fam = fam.rstrip()
		f = []
		try:
			f = open(peptide_dir_name+"/"+fam+"/"+fam+"_conserved_peptides.txt").readlines()
		except:	
			f = open(peptide_dir_name+"/"+fam+"/"+fam+"_peptides_scores.txt").readlines()
		for x in range(len(f)):
			line = f[x].rstrip()
			if "group" in line:
				g = Group(fam, line.split("group ")[-1])
				pep_array = f[x+1].split(";")
				pep_array.pop()
				for pep in pep_array:
					arr = pep.split(",")
					g.pep_hash[arr[0]] = float(arr[-1])
					fam_all_peps_array.append(arr[0])
				pep_group_array.append(g)
		set(fam_all_peps_array)
		prot_hit_array = []
		for p in prot_array:
			prot_fam_peps = set.intersection(set(p.peptides), set(fam_all_peps_array))
			no_peps = len(prot_fam_peps)
			group_score_array = []
			if no_peps == hit_cut_off:
				index_arr = []
				for pep in prot_fam_peps:
					index_arr.append(p.seq.find(pep))
				index_arr.sort()
				if index_arr[-1]-index_arr[0] > 3:
					no_peps += 1
			if no_peps > hit_cut_off and len(set(list("".join(prot_fam_peps)))) > 4:
				for g in pep_group_array:
					pep_array = set.intersection(set(prot_fam_peps), set(g.pep_hash.keys()))
					hits = len(pep_array)
					freq = 0.0
					for pep in pep_array:
						if hits >= hit_cut_off:
							freq += g.pep_hash[pep]
						if freq >= freq_cut_off:
							group_score_array.append([freq, hits, g.number, pep_array])
			if len(group_score_array) > 0:
				group_score_array.sort(key= lambda x: (int(x[0]), int(x[1]), -int(x[2])))
				if group_score_array[-1][1] >= hit_cut_off and group_score_array[-1][0] >= freq_cut_off:
					if protein_hits == 0:
						out.write("Family "+fam+"\n")
					out.write(p.name+'\n')
					out.write(p.seq+'\n')
					out.write(",".join(group_score_array[-1][3])+'\n')
					out.write(str(group_score_array[-1][1])+'\n')
					out.write(str(float(str(group_score_array[-1][0])[0:4]))+'\n')
					out.write(group_score_array[-1][2]+'\n')
					protein_hits += 1
					
call(['cp', outf_name, 'temp'])
			



#!/usr/bin/env python3
##################################################
# CGCFinder v4
#
# Rewriten for dbCAN2
#
# Written by Le Huang under the supervision of
# Dr. Yin at NIU
#
#
# Last updated 12/24/18
#-updating info
#-adding stp
##################################################


import argparse
import csv
import sys


#set up argument parser
parser = argparse.ArgumentParser(description='CAZyme Gene Cluster Finder')

parser.add_argument('gffFile', help='GFF file containing genome information')
parser.add_argument('--distance', '-d', type=int, choices=[0,1,2,3,4,5,6,7,8,9,10], default=2, help='The distance allowed between two signature genes')
parser.add_argument('--siggenes', '-s', choices=['all', 'tp', 'tf','stp','tp+tf','tp+stp','tf+stp'], default='all', help='Signature genes types required. all=CAZymes,TC,TF; tp=CAZymes,TC; tf=CAZymes,TF')
parser.add_argument('--output', '-o', help='Output file name')

args = parser.parse_args()


#open output file
out = open(args.output, 'w+')

#global vars
cluster = [0, 0, 0, 0] #cazyme, tp, tf, stp
num_clusters = 0

#define boolean function to determine if a cluster meets cluster requirements
def isCluster(): 
	global cluster
	if args.siggenes == 'all':
		if cluster[0] > 0 and cluster[1] > 0 and cluster[2] > 0 and cluster[3]:
			return True
	elif args.siggenes == 'tf':
		if cluster[0] > 0 and cluster[2] > 0:
			return True
	elif args.siggenes == 'tp':
		if cluster[0] > 0 and cluster[1] > 0:
			return True
	elif args.siggenes == 'stp':
		if cluster[0] > 0 and cluster[3] > 0:
			return True
	elif args.siggenes == 'tp+tf':
		if cluster[0] > 0 and cluster[1] > 0 and cluster[2]>0:
			return True
	elif args.siggenes == 'tp+stp':
		if cluster[0] > 0 and cluster[1] > 0 and cluster[3]>0:
			return True
	elif args.siggenes == 'tf+stp':
		if cluster[0] > 0 and cluster[2] > 0 and cluster[3]>0:
			return True
	else:
		print('Warning: invalid siggenes argument')
	return False

#define boolean function to detemine if a gene is important (a signature gene)
def isImportant(gene):
	if gene == 'CAZyme':
		return True
	else:
		if gene == 'TC' and (args.siggenes == 'tp' or args.siggenes == 'all' or args.siggenes == 'tp+tf' or args.siggenes == 'tp+stp'):
			return True
		if gene == 'TF' and (args.siggenes == 'tf' or args.siggenes == 'all' or args.siggenes == 'tp+tf' or args.siggenes == 'tf+stp'):
			return True
		if gene == 'STP' and (args.siggenes == 'stp' or args.siggenes == 'all'or args.siggenes == 'tp+stp' or args.siggenes == 'tf+stp' ):
			return True
	return False

def isSigGene(gene):
	if gene == 'CAZyme' or gene == 'TC' or gene == 'TF' or gene == 'STP':
		return True
	else:
		return False
		
#define function to increase the cluster count
def increaseClusterCount(gene):
	global cluster
	if gene == 'CAZyme':
		cluster[0] += 1
	elif gene == 'TC':
		cluster[1] += 1
	elif gene == 'TF':
		cluster[2] += 1
	elif gene == 'STP':
		cluster[3] += 1
	else:
		print("Warning: increaseClusterCount was called on bad functional domain")

#define function to search for a cluster once an important gene has been found
#this function also handles output
def startSearch(startRow, contig):
	global cluster
	global num_clusters
	dis = args.distance
	index = startRow
	between = 0
	lastImportant = 0
	while index < len(contig):
		index += 1
		fd = contig[index][2]
		if isImportant(fd):
			increaseClusterCount(fd)
			lastImportant = index
			between = 0
		else:
			between += 1
		if between > dis or index >= (len(contig)-1):
			if isCluster():
				num_clusters += 1
				#output file columns
				#geneNumber type[2] downDis upDis CGC# contig[0] geneStart[3] geneEnd[4] geneID[8,ID] direc[6] note[8]
				for j in range(startRow, lastImportant + 1):
					fd = contig[j][2]
					if isSigGene(fd):
						upDown = findNear(contig, j)
						notes = contig[j][8].split(";")
						ID= ""
						for note in notes:
							if "ID" in note:
								ID = note.split("=")[1]
						row = [str(j), fd, str(upDown[1]), str(upDown[0]), 'CGC'+str(num_clusters), contig[j][0], contig[j][3], contig[j][4], ID, contig[j][6], contig[j][8]]
					else:
						row = [str(j), 'null', 'null', 'null', 'CGC'+str(num_clusters), contig[j][0], contig[j][3], contig[j][4], ID, contig[j][6]]
					try:
						row.append(contig[j][8])
					except:
						pass
					out.write('\t'.join(row) + '\n')
				out.write('+++++' + '\n')
			cluster = [0, 0, 0, 0]
			return index
			
#define function to find how close important genes are to each other
def findNear(contig, index):
	vals = ['null', 'null']
	k = index - 1
	l = index + 1
	while k >= 0:
		if isImportant(contig[k][2]):
			vals[0] = index - k - 1
			break
		else:
			k -= 1
	while l <= len(contig) - 1:
		if isImportant(contig[l][2]):
			vals[1] = l - index - 1
			break
		else:
			l += 1
	return vals
		

#load contig into an array 
contigs = {}
with open(args.gffFile) as f:
	for line in f:
		row = line.rstrip().split('\t')
		if row[0] not in contigs:
			contigs[row[0]] = []
		contigs[row[0]].append(row)

#loop through contig
for key in contigs:
	contig = contigs[key]
	num_clusters = 0
	i = 0
	while i < len(contig) - 1:
		fd = contig[i][2]
		
		if isImportant(fd):
			increaseClusterCount(fd)
			i = startSearch(i, contig)
		else:
			i += 1
			
if args.output != 'none':
	out.close()



#!/usr/bin/env python3
from multiprocessing.dummy import Pool as ThreadPool
from subprocess import call
import sys
import natsort
import os
import os.path
import re

from Hotpep.hotpep_data import hotpep_data_path

protein_dir_name = hotpep_data_path("fungus_fungus")
if len(sys.argv) > 2:
	protein_dir_name = sys.argv[2].replace("?", " ")
peptide_dir_name = hotpep_data_path("CAZY_PPR_patterns/GH")
if len(sys.argv) > 3:
	peptide_dir_name = sys.argv[3].replace("?", " ")

threads = 8
peptide_length = 6 #length of conserved peptides
hit_cut_off = 3 #number of conserved peptides necessary to classify a protein
freq_cut_off = 1.0 #minimum sum of frequencies necessary to classify a protein

if len(sys.argv) > 1:
	threads = int(sys.argv[1])
if len(sys.argv) > 4:
	peptide_length = int(sys.argv[4])
if len(sys.argv) > 5:
	hit_cut_off = int(sys.argv[5])
if len(sys.argv) > 6:
	freq_cut_off = float(sys.argv[6])
variables = [ protein_dir_name.replace(" ", "?"), peptide_dir_name.replace(" ", "?"), peptide_length, hit_cut_off, freq_cut_off ]

class Protein:
	def __init__(self, sequ):
		self.seq = sequ.upper()
		self.dna = None
		self.name = None
		self.peptides = None
		self.hits = None
		self.freq = None
		self.group = None
		self.subp = None
		self.accession = None
		self.neighbour_seqs = None
		self.ec = None #added by Le Feb 19, 2020
		
	
def callCustom(args):
	return call(args, shell=True)
	
print ("Assigning proteins to groups")
args_array = []
var1 = 1
varlist = " ".join(str(x) for x in variables)
pool = ThreadPool(threads)
while var1 <= threads:
	args_array.append(("bact_group_many_proteins_many_patterns.py "+ str(var1) + " " + varlist))
	var1 += 1
pool.map(callCustom, args_array)

print("Collecting Results")

pep_list_array = []
try:
	f = open(peptide_dir_name+"/large_fams.txt", 'r')
except:
	f = open(peptide_dir_name+"/fam_list.txt", 'r')
for line in f:
	pep_list_array.append(line.rstrip())
f.close()
pep_list_hash = {}
for fam in pep_list_array:
	pep_list_hash[fam]=[]
natsort.natsorted(pep_list_array)
var1 = 1
fam = ""
while var1 <= threads:
	f = open(protein_dir_name+"/thread"+str(var1)+".txt", 'r').readlines()
	for x in range(len(f)):
		line = f[x].rstrip()
		if line.startswith("Family"):
			fam = line.split(" ")[-1]
		elif line.startswith(">"):
			p = Protein(f[x+1].rstrip())
			p.name = line
			p.peptides = f[x+2].rstrip()
			p.hits = int(f[x+3].rstrip())
			p.freq = float(f[x+4].rstrip())
			p.group = int(f[x+5].rstrip())
			#add by Le start Feb 19, 2020
			fam_main = " ".join(re.findall("[a-zA-Z]+", fam))
			if os.path.exists(f"CAZY_PPR_patterns/{fam_main}/{fam}/{fam}_group_ec.txt"):
				kk = open(f"CAZY_PPR_patterns/{fam_main}/{fam}/{fam}_group_ec.txt").\
						readlines()[p.group-1].rstrip().split("\t")
				if len(kk) >1:
					p.ec = kk[1]
				else:
					p.ec = "NA"
			else:
				p.ec = "NA"
			#add by Le end Feb 19, 2020
			pep_list_hash[fam].append(p)
	var1 += 1		
output_dir_name = protein_dir_name+"/Results"
if not os.path.exists(output_dir_name):
	call(["mkdir", output_dir_name])
for fam in pep_list_array:
	hit_array = pep_list_hash[fam]
	#if len(hit_array) > 0:
	fam_file = open(output_dir_name+"/output.txt", "a")
	hit_array.sort(key= lambda x: (x.group, -x.freq, -x.hits))
	for p in hit_array:
		#added ec number by Le start
		fam_file.write(fam+ '\t' +str(p.group)+"\t"+p.name.split(' ')[0][1:]+"\t"+str(p.freq)+"\t"+str(p.hits)+"\t"+p.peptides+"\t"+p.ec+"\n")
		#added ec number by Le end
	fam_file.close()




#!/usr/bin/env python
#
# distutils setup script for dbcan package
# also installs the bundled Hotpep scripts and data files

from glob import glob
from os import listdir
from os.path import isfile
from setuptools import setup, find_packages

long_description = """This is the standalone version of dbCAN annotation tool for automated CAZyme annotation (known as run_dbCAN.py), written by Tanner Yohe and Le Huang.
"""

setup(name='run_dbcan',
      version="2.0.9",
    #   use_scm_version=True,
      setup_requires=['setuptools_scm', 'setuptools_scm_git_archive'],
      description='Standalone version of dbCAN annotation tool for automated CAZyme annotation',
      long_description=long_description,
      author='Tanner Yohe and Le Huang',
      author_email='lehuang@unc.edu',
      url='https://github.com/linnabrown/run_dbcan',
      classifiers=[
          'Development Status :: 4 - Beta',
          'Environment :: Console',
          'Intended Audience :: Science/Research',
          'License :: OSI Approved :: GNU General Public License v3 (GPLv3)',
          'Operating System :: POSIX :: Linux',
          'Programming Language :: Python :: 3',
          'Topic :: Scientific/Engineering :: Bio-Informatics',
      ],
      packages=find_packages(),
      include_package_data = True,
      package_data={
          'Hotpep': ['*.txt'],
      },

      scripts=[
          'CGCFinder.py',
          'hmmscan-parser.py',
          'run_dbcan.py',
          'Hotpep/add_functions_orf.py',
          'Hotpep/bact_group_many_proteins_many_patterns.py',
          'Hotpep/list_multidomain_proteins.py',
          'Hotpep/parallel_group_many_proteins_many_patterns_noDNA.py',
          'Hotpep/train_many_organisms_many_families.py',
      ],
      license='GPLv3',
      install_requires=[
          'natsort',
          'setuptools'
      ],
      python_requires='>=3',
     )
